<?php
/**
 * Overrides default one page checkout with Flash Checkout
 *
 * Flash Checkout by Kahanit(https://www.kahanit.com/) is licensed under a
 * Creative Creative Commons Attribution-NoDerivatives 4.0 International License.
 * Based on a work at https://www.kahanit.com/.
 * Permissions beyond the scope of this license may be available at https://www.kahanit.com/.
 * To view a copy of this license, visit http://creativecommons.org/licenses/by-nd/4.0/.
 *
 * @author    Amit Sidhpura <amit@kahanit.com>
 * @copyright 2015 Kahanit
 * @license   http://creativecommons.org/licenses/by-nd/4.0/
 * @version   1.0.1.0
 */

/**
 * Class OrderOpcController
 */
class OrderOpcController extends ParentOrderController
{
    public $php_self = 'order-opc';

    public $isLogged;

    protected $ajax_refresh = false;

    protected $ajax_panel = true;

    protected $module_name = 'flashcheckout';

    protected $module_path = null;

    protected $_address;

    public $successes = array();

    public $warnings = array();

    public $errors = array();

    public $datas = array();

    /* inherited abstract  methods */

    public function init()
    {
        parent::init();

        $this->module_path = __PS_BASE_URI__.'modules/'.$this->module_name.'/';
        $ajax = Tools::getValue('ajax', false);

        if ($ajax) {
            $method = Tools::getValue('method', false);
            header('Content-Type: application/json');

            if ($method !== false && method_exists($this, $method)) {
                $this->$method();
            }
        }
    }

    public function setMedia()
    {
        parent::setMedia();

        /* include css */
        $this->addCSS($this->module_path.'views/css/pnotify.core.min.css');
        $this->addCSS($this->module_path.'views/css/flashcheckout.css');

        /* include js */
        unset($this->context->controller->js_files[array_search(_THEME_JS_DIR_.'cart-summary.js', $this->context->controller->js_files)]);
        unset($this->context->controller->js_files[array_search(_THEME_JS_DIR_.'order-address.js', $this->context->controller->js_files)]);

        $this->addJquery();
        $this->addJS(_PS_JS_DIR_.'validate.js');
        $this->addJS($this->module_path.'views/js/pnotify.core.min.js');
        $this->addJS($this->module_path.'views/js/flashcheckout.js');
    }

    public function initContent()
    {
        parent::initContent();
        Tools::safePostVars();

        $this->getFlashCheckout(false);
        /*header('Content-Type: application/json');
        echo(json_encode($this->context->smarty->getTemplateVars()));
        die();*/
    }

    public function displayAjax()
    {
		echo Tools::jsonEncode($this->getFlashCheckout(true));
        die();
    }

    public function setTemplate($template)
    {
        if (!$path = $this->getTemplatePath($template)) {
            throw new PrestaShopException("Template '$template' not found");
        }

        $this->template = $path;
    }

    public function getTemplatePath($template)
    {
        if (Tools::file_exists_cache(_PS_THEME_DIR_.'modules/'.$this->module_name.'/'.$template)) {
            return _PS_THEME_DIR_.'modules/'.$this->module_name.'/'.$template;
        } elseif (Tools::file_exists_cache(_PS_THEME_DIR_.'modules/'.$this->module_name.'/views/templates/front/'.$template)) {
            return _PS_THEME_DIR_.'modules/'.$this->module_name.'/views/templates/front/'.$template;
        } elseif (Tools::file_exists_cache(_PS_MODULE_DIR_.$this->module_name.'/views/templates/front/'.$template)) {
            return _PS_MODULE_DIR_.$this->module_name.'/views/templates/front/'.$template;
        }

        return false;
    }

    /* common methods */

    protected function getFlashCheckout($ajax = false)
    {
        if (!$ajax) {
            $this->getFlashCheckoutSummary(false);
            $this->getFlashCheckoutDelivery(false);
            $this->getFlashCheckoutPayment(false);
            $this->getFlashCheckoutAccount(false);

            $this->setTemplate('checkout.tpl');

            return true;
        } else {
            $return = array_merge(
                array(
                    'hasSuccess' => count($this->successes) > 0,
                    'successes' => $this->successes,
                    'hasWarning' => count($this->warnings) > 0,
                    'warnings' => $this->warnings,
                    'hasError' => count($this->errors) > 0,
                    'errors' => $this->errors
                ),
                $this->datas
            );

            if ($this->ajax_panel) {
                $return = array_merge(
                    $return,
                    array(
                        'summary-panel' => $this->getFlashCheckoutSummary(true),
                        'delivery-panel' => $this->getFlashCheckoutDelivery(true),
                        'payment-panel' => $this->getFlashCheckoutPayment(true),
                        'account-panel' => $this->getFlashCheckoutAccount(true)
                    )
                );
            }

            return $return;
        }
    }

    protected function getFlashCheckoutSummary($fetch = false)
    {
        $this->_assignSummaryInformations();
        $this->context->smarty->assign('advanced_payment_api', (bool)Configuration::get('PS_ADVANCED_PAYMENT_API'));

        /* fetch template */
        if ($fetch) {
            $this->setTemplate('summary/summary-panel.tpl');

            return $this->context->smarty->fetch($this->template);
        }

        return true;
    }

    protected function getFlashCheckoutDelivery($fetch = false)
    {
        /* id_carrier is not defined in database before choosing a carrier,
        set it to a default one to match a potential cart _rule */
        if (empty($this->context->cart->id_carrier)) {
            $checked = $this->context->cart->simulateCarrierSelectedOutput();
            $checked = ((int)Cart::desintifier($checked));
            $this->context->cart->id_carrier = $checked;
            $this->context->cart->update();
            CartRule::autoRemoveFromCart($this->context);
            CartRule::autoAddToCart($this->context);
        }

        /* wrapping and tos */
        $this->_assignWrappingAndTOS();

        /* If a rule offer free-shipping, force hidding shipping prices */
        $free_shipping = false;

        foreach ($this->context->cart->getCartRules() as $rule) {
            if ($rule['free_shipping'] && !$rule['carrier_restriction']) {
                $free_shipping = true;
                break;
            }
        }

        $this->context->smarty->assign('free_shipping', $free_shipping);
        $this->context->smarty->assign('errorCarrier', Tools::displayError('You must choose a carrier.', false));
        $this->_assignCarrier();

        /* fetch template */
        if ($fetch) {
            $this->setTemplate('delivery/delivery-panel.tpl');

            return $this->context->smarty->fetch($this->template);
        }

        return true;
    }

    protected function getFlashCheckoutPayment($fetch = false)
    {
        $this->context->smarty->assign('isPaymentStep', Tools::getIsset('isPaymentStep') && Tools::getValue('isPaymentStep'));
        $this->_assignPayment();
        $this->context->smarty->assign('id_payment', $this->context->cookie->id_payment);
        $this->context->smarty->assign('errorTOS', Tools::displayError('You must accept the Terms of Service.', false));

        /* fetch template */
        if ($fetch) {
            $this->setTemplate('payment/payment-panel.tpl');

            return $this->context->smarty->fetch($this->template);
        }

        return true;
    }

    protected function getFlashCheckoutAccount($fetch = false)
    {
        /* if cart is virtual set delivery address as invoice address */
        if ($this->context->cart->isVirtualCart() && ($this->isLogged || $this->context->cookie->is_guest)) {
            if (isset($this->context->cart->id_address_delivery) && isset($this->context->cart->id_address_invoice) &&
                $this->context->cart->id_address_delivery !== $this->context->cart->id_address_invoice
            ) {
                $this->context->cart->id_address_delivery = $this->context->cart->id_address_invoice;

                if (!$this->context->cart->update()) {
                    $this->errors[] = Tools::displayError('An error occurred while updating your cart.');
                }
            }
        }

        if (Configuration::get('PS_RESTRICT_DELIVERED_COUNTRIES')) {
            $countries = Carrier::getDeliveredCountries($this->context->language->id, true, true);
        } else {
            $countries = Country::getCountries($this->context->language->id, true);
        }

        $this->context->smarty->assign('isGuest', isset($this->context->cookie->is_guest) ? $this->context->cookie->is_guest : 0);
        $this->context->smarty->assign('PS_GUEST_CHECKOUT_ENABLED', Configuration::get('PS_GUEST_CHECKOUT_ENABLED'));
        $this->context->smarty->assign('countries', $countries);
        $this->context->smarty->assign('sl_country', (int)Tools::getCountry());
        $this->context->smarty->assign('genders', Gender::getGenders());
        $this->context->smarty->assign('one_phone_at_least', (int)Configuration::get('PS_ONE_PHONE_AT_LEAST'));
        $this->context->smarty->assign('HOOK_CREATE_ACCOUNT_FORM', Hook::exec('displayCustomerAccountForm'));
        $this->context->smarty->assign('HOOK_CREATE_ACCOUNT_TOP', Hook::exec('displayCustomerAccountFormTop'));
        $this->context->smarty->assign('years', Tools::dateYears());
        $this->context->smarty->assign('months', Tools::dateMonths());
        $this->context->smarty->assign('days', Tools::dateDays());

        if ($this->isLogged || $this->context->cookie->is_guest) {
            if ($this->context->customer->birthday) {
                $birthday = explode('-', $this->context->customer->birthday);
            } else {
                $birthday = array('0', '0', '0');
            }

            $customer = $this->context->customer;
            $customer->sl_year = $birthday[0];
            $customer->sl_month = $birthday[1];
            $customer->sl_day = $birthday[2];

            $this->context->smarty->assign('customer', $customer);
        }

        $this->context->smarty->assign('newsletter', Configuration::get('PS_CUSTOMER_NWSL') || (Module::isInstalled('blocknewsletter') && Module::getInstanceByName('blocknewsletter')->active));
        $this->context->smarty->assign('optin', (bool)Configuration::get('PS_CUSTOMER_OPTIN'));
        $this->context->smarty->assign('field_required', $this->context->customer->validateFieldsRequiredDatabase());

        if ($this->isLogged) {
            $this->_assignAddress();
        }

        $this->_processAddressFormat();
        $this->assignCountries();
        $this->assignVatNumber();

        // Assign common vars
        $this->context->smarty->assign(array(
            'address_validation' => Address::$definition['fields'],
            'one_phone_at_least' => (int)Configuration::get('PS_ONE_PHONE_AT_LEAST'),
            'onr_phone_at_least' => (int)Configuration::get('PS_ONE_PHONE_AT_LEAST'),
            'ajaxurl' => _MODULE_DIR_,
            'errors' => $this->errors,
            'token' => Tools::getToken(false),
            'select_address' => (int)Tools::getValue('select_address'),
            'address' => $this->_address,
            'id_address' => (Validate::isLoadedObject($this->_address)) ? $this->_address->id : 0
        ));

        if ($back = Tools::getValue('back')) {
            $this->context->smarty->assign('back', Tools::safeOutput($back));
        }

        if ($mod = Tools::getValue('mod')) {
            $this->context->smarty->assign('mod', Tools::safeOutput($mod));
        }

        if (isset($this->context->cookie->account_created)) {
            $this->context->smarty->assign('account_created', 1);
            unset($this->context->cookie->account_created);
        }

        /* fetch template */
        if ($fetch) {
            $this->setTemplate('account/account-panel.tpl');

            return $this->context->smarty->fetch($this->template);
        }

        return true;
    }

    /* address controller methods */

    protected function assignCountries()
    {
        if (Configuration::get('PS_RESTRICT_DELIVERED_COUNTRIES')) {
            $countries = Carrier::getDeliveredCountries($this->context->language->id, true, true);
        } else {
            $countries = Country::getCountries($this->context->language->id, true);
        }

        // Assign vars
        $this->context->smarty->assign(array(
            'countries' => $countries
        ));
    }

    protected function assignVatNumber()
    {
        $vat_number_exists = file_exists(_PS_MODULE_DIR_.'vatnumber/vatnumber.php');
        $vat_number_management = Configuration::get('VATNUMBER_MANAGEMENT');
        if ($vat_number_management && $vat_number_exists) {
            include_once(_PS_MODULE_DIR_.'vatnumber/vatnumber.php');
        }

        if ($vat_number_management && $vat_number_exists && VatNumber::isApplicable((int)Tools::getCountry())) {
            $vat_display = 2;
        } elseif ($vat_number_management) {
            $vat_display = 1;
        } else {
            $vat_display = 0;
        }

        $this->context->smarty->assign(array(
            'vatnumber_ajax_call' => file_exists(_PS_MODULE_DIR_.'vatnumber/ajax.php'),
            'vat_display' => $vat_display,
        ));
    }

    /* summary methods */

    protected function plusQuantity()
    {
        $this->updateQuantity('up');
    }

    protected function minusQuantity()
    {
        $this->updateQuantity('down');
    }

    protected function updateQuantity($operator)
    {
        $id_product = Tools::getValue('id_product', 0);
        $id_product_attribute = Tools::getValue('ipa', null);
        $id_customization = Tools::getValue('id_customization', false);
        $id_address_delivery = Tools::getValue('id_address_delivery', 0);
        $qty = Tools::getValue('qty', 0);

        if ($id_product !== 0) {
            if ($this->context->cart->updateQty($qty, $id_product, $id_product_attribute, $id_customization, $operator, $id_address_delivery)) {
                $this->successes[] = 'Quantity updated successfully';

                return;
            }
        }

        $this->errors[] = 'Error updating quantity';
    }

    protected function addCoupon()
    {
        $code = Tools::getValue('code', '');

        if ($code !== '' && $this->context->cart->addCartRule(CartRuleCore::getIdByCode($code))) {
            $this->successes[] = 'Coupon added successfully';
        } else {
            $this->errors[] = 'Error adding coupon';
        }
    }

    protected function removeCoupon()
    {
        $id_cart_rule = Tools::getValue('id_cart_rule', 0);

        if ($id_cart_rule !== 0 && $this->context->cart->removeCartRule($id_cart_rule)) {
            $this->successes[] = 'Coupon removed successfully';
        } else {
            $this->errors[] = 'Error removing coupon';
        }
    }

    /* address methods */

    protected function changeAddress()
    {
        if ($this->context->customer->isLogged(true)) {

            $address_delivery = new Address((int)Tools::getValue('id_address_delivery'));
            $address_invoice = ((int)Tools::getValue('id_address_delivery') == (int)Tools::getValue('id_address_invoice')
                ? $address_delivery : new Address((int)Tools::getValue('id_address_invoice')));

            if ($address_delivery->id_customer != $this->context->customer->id || $address_invoice->id_customer != $this->context->customer->id) {
                $this->errors[] = Tools::displayError('This address is not yours.');
            } elseif (!Address::isCountryActiveById((int)Tools::getValue('id_address_delivery'))) {
                $this->errors[] = Tools::displayError('This address is not in a valid area.');
            } elseif (!Validate::isLoadedObject($address_delivery) || !Validate::isLoadedObject($address_invoice) || $address_invoice->deleted || $address_delivery->deleted) {
                $this->errors[] = Tools::displayError('This address is invalid.');
            } else {
                $this->context->cart->id_address_delivery = (int)Tools::getValue('id_address_delivery');
                $this->context->cart->id_address_invoice = Tools::isSubmit('same')
                    ? $this->context->cart->id_address_delivery : (int)Tools::getValue('id_address_invoice');

                if (!$this->context->cart->update()) {
                    $this->errors[] = Tools::displayError('An error occurred while updating your cart.');
                }

                $infos = Address::getCountryAndState((int)$this->context->cart->id_address_delivery);

                if (isset($infos['id_country']) && $infos['id_country']) {
                    $country = new Country((int)$infos['id_country']);
                    $this->context->country = $country;
                }

                // Address has changed, so we check if the cart rules still apply
                $cart_rules = $this->context->cart->getCartRules();
                CartRule::autoRemoveFromCart($this->context);
                CartRule::autoAddToCart($this->context);

                if ((int)Tools::getValue('allow_refresh')) {
                    // If the cart rules has changed, we need to refresh the whole cart
                    $cart_rules2 = $this->context->cart->getCartRules();

                    if (count($cart_rules2) != count($cart_rules)) {
                        $this->ajax_refresh = true;
                    } else {
                        $rule_list = array();

                        foreach ($cart_rules2 as $rule) {
                            $rule_list[] = $rule['id_cart_rule'];
                        }

                        foreach ($cart_rules as $rule) {
                            if (!in_array($rule['id_cart_rule'], $rule_list)) {
                                $this->ajax_refresh = true;
                                break;
                            }
                        }
                    }
                }

                if (!$this->context->cart->isMultiAddressDelivery()) {
                    $this->context->cart->setNoMultishipping();
                } // As the cart is no multishipping, set each delivery address lines with the main delivery address

                if (!count($this->errors)) {
                    $this->successes[] = 'Address updated successfully';
                }
            }
        }
    }

    protected function deleteAddress()
    {
        $id_address = 0;

        if ($this->ajax && Tools::isSubmit('type')) {
            if (Tools::getValue('type') == 'delivery' && isset($this->context->cart->id_address_delivery)) {
                $id_address = (int)$this->context->cart->id_address_delivery;
            } elseif (Tools::getValue('type') == 'invoice' && isset($this->context->cart->id_address_invoice)
                && $this->context->cart->id_address_invoice != $this->context->cart->id_address_delivery
            ) {
                $id_address = (int)$this->context->cart->id_address_invoice;
            }
        }

        // Initialize address
        if ($id_address) {
            $address = new Address($id_address);

            if (Validate::isLoadedObject($address) && Customer::customerHasAddress($this->context->customer->id, $id_address)) {
                if ($address->delete()) {
                    if ($this->context->cart->id_address_invoice == $address->id) {
                        unset($this->context->cart->id_address_invoice);
                    }

                    if ($this->context->cart->id_address_delivery == $address->id) {
                        unset($this->context->cart->id_address_delivery);
                        $this->context->cart->updateAddressId(
                            $address->id,
                            (int)Address::getFirstCustomerAddressId(Context::getContext()->customer->id)
                        );
                    }

                    $this->successes[] = 'Address deleted successfully';

                    return;
                }
            }
        }

        $this->errors[] = 'Error deleting address';
    }

    /* delivery methods */

    protected function updateCarrier()
    {
        if ((Tools::isSubmit('delivery_option') || Tools::isSubmit('id_carrier')) && Tools::isSubmit('recyclable')
            && Tools::isSubmit('gift') && Tools::isSubmit('gift_message')
        ) {
            if ($this->_processCarrier()) {
                $this->successes[] = 'Carrier updated successfully';

                return;
            }
        }

        $this->errors[] = 'Error updating carrier';
    }

    protected function updateMessage()
    {
        $this->ajax_panel = false;

        if (Tools::isSubmit('message')) {
            $txt_message = urldecode(Tools::getValue('message', ''));
            $this->_updateMessage($txt_message);
            $message = Message::getMessageByCartId((int)$this->context->cart->id);

            $this->datas['message'] = $message['message'];
        }
    }

    /* payment methods */

    protected function updatePayment()
    {
        $id_payment = Tools::getValue('id_payment', '');

        if ($id_payment !== '') {
            $this->context->cookie->__set('id_payment', $id_payment);
            $this->successes[] = 'Payment method updated successfully';

            return;
        }

        $this->errors[] = 'Error updating payment method';
    }

    protected function confirmOrder()
    {
        $this->context->cookie->checkedTOS = (int)Tools::getValue('agreeterms', 0);
        $this->ajax_panel = false;

        if (!$this->context->cookie->checkedTOS && Configuration::get('PS_CONDITIONS')) {
            $this->errors[] = Tools::displayError('Please accept the Terms of Service.');

            return;
        }

        if (($id_order = $this->_checkFreeOrder()) && $id_order) {
            $order = new Order((int)$id_order);

            /* If guest we clear the cookie for security reason */
            if ($this->context->customer->is_guest) {
                $this->context->customer->logout();
            }
            /* (end) If guest we clear the cookie for security reason */

            $this->datas['order_isfree'] = true;
            $this->datas['order_reference'] = $order->reference;
            $this->datas['order_email'] = $this->context->customer->email;
        }
    }

    /* default order opc methods */

    protected function _getGuestInformations()
    {
        $customer = $this->context->customer;
        $address_delivery = new Address($this->context->cart->id_address_delivery);

        $id_address_invoice = $this->context->cart->id_address_invoice != $this->context->cart->id_address_delivery ? (int)$this->context->cart->id_address_invoice : 0;
        $address_invoice = new Address($id_address_invoice);

        if ($customer->birthday) {
            $birthday = explode('-', $customer->birthday);
        } else {
            $birthday = array('0', '0', '0');
        }

        return array(
            'id_customer' => (int)$customer->id,
            'email' => Tools::htmlentitiesUTF8($customer->email),
            'customer_lastname' => Tools::htmlentitiesUTF8($customer->lastname),
            'customer_firstname' => Tools::htmlentitiesUTF8($customer->firstname),
            'newsletter' => (int)$customer->newsletter,
            'optin' => (int)$customer->optin,
            'id_address_delivery' => (int)$this->context->cart->id_address_delivery,
            'company' => Tools::htmlentitiesUTF8($address_delivery->company),
            'lastname' => Tools::htmlentitiesUTF8($address_delivery->lastname),
            'firstname' => Tools::htmlentitiesUTF8($address_delivery->firstname),
            'vat_number' => Tools::htmlentitiesUTF8($address_delivery->vat_number),
            'dni' => Tools::htmlentitiesUTF8($address_delivery->dni),
            'address1' => Tools::htmlentitiesUTF8($address_delivery->address1),
            'postcode' => Tools::htmlentitiesUTF8($address_delivery->postcode),
            'city' => Tools::htmlentitiesUTF8($address_delivery->city),
            'phone' => Tools::htmlentitiesUTF8($address_delivery->phone),
            'phone_mobile' => Tools::htmlentitiesUTF8($address_delivery->phone_mobile),
            'id_country' => (int)$address_delivery->id_country,
            'id_state' => (int)$address_delivery->id_state,
            'id_gender' => (int)$customer->id_gender,
            'sl_year' => $birthday[0],
            'sl_month' => $birthday[1],
            'sl_day' => $birthday[2],
            'company_invoice' => Tools::htmlentitiesUTF8($address_invoice->company),
            'lastname_invoice' => Tools::htmlentitiesUTF8($address_invoice->lastname),
            'firstname_invoice' => Tools::htmlentitiesUTF8($address_invoice->firstname),
            'vat_number_invoice' => Tools::htmlentitiesUTF8($address_invoice->vat_number),
            'dni_invoice' => Tools::htmlentitiesUTF8($address_invoice->dni),
            'address1_invoice' => Tools::htmlentitiesUTF8($address_invoice->address1),
            'address2_invoice' => Tools::htmlentitiesUTF8($address_invoice->address2),
            'postcode_invoice' => Tools::htmlentitiesUTF8($address_invoice->postcode),
            'city_invoice' => Tools::htmlentitiesUTF8($address_invoice->city),
            'phone_invoice' => Tools::htmlentitiesUTF8($address_invoice->phone),
            'phone_mobile_invoice' => Tools::htmlentitiesUTF8($address_invoice->phone_mobile),
            'id_country_invoice' => (int)$address_invoice->id_country,
            'id_state_invoice' => (int)$address_invoice->id_state,
            'id_address_invoice' => $id_address_invoice,
            'invoice_company' => Tools::htmlentitiesUTF8($address_invoice->company),
            'invoice_lastname' => Tools::htmlentitiesUTF8($address_invoice->lastname),
            'invoice_firstname' => Tools::htmlentitiesUTF8($address_invoice->firstname),
            'invoice_vat_number' => Tools::htmlentitiesUTF8($address_invoice->vat_number),
            'invoice_dni' => Tools::htmlentitiesUTF8($address_invoice->dni),
            'invoice_address' => $this->context->cart->id_address_invoice !== $this->context->cart->id_address_delivery,
            'invoice_address1' => Tools::htmlentitiesUTF8($address_invoice->address1),
            'invoice_address2' => Tools::htmlentitiesUTF8($address_invoice->address2),
            'invoice_postcode' => Tools::htmlentitiesUTF8($address_invoice->postcode),
            'invoice_city' => Tools::htmlentitiesUTF8($address_invoice->city),
            'invoice_phone' => Tools::htmlentitiesUTF8($address_invoice->phone),
            'invoice_phone_mobile' => Tools::htmlentitiesUTF8($address_invoice->phone_mobile),
            'invoice_id_country' => (int)$address_invoice->id_country,
            'invoice_id_state' => (int)$address_invoice->id_state,
        );
    }

    protected function _assignCarrier()
    {
        if (!$this->isLogged) {
            $carriers = $this->context->cart->simulateCarriersOutput();
            $old_message = Message::getMessageByCartId((int)$this->context->cart->id);
            $this->context->smarty->assign(array(
                'HOOK_EXTRACARRIER' => null,
                'HOOK_EXTRACARRIER_ADDR' => null,
                'oldMessage' => isset($old_message['message']) ? $old_message['message'] : '',
                'HOOK_BEFORECARRIER' => Hook::exec('displayBeforeCarrier', array(
                    'carriers' => $carriers,
                    'checked' => $this->context->cart->simulateCarrierSelectedOutput(),
                    'delivery_option_list' => $this->context->cart->getDeliveryOptionList(),
                    'delivery_option' => $this->context->cart->getDeliveryOption(null, true)
                ))
            ));
        } else {
            parent::_assignCarrier();
        }
    }

    protected function _assignPayment()
    {
        $this->context->smarty->assign(array(
            'HOOK_TOP_PAYMENT' => ($this->isLogged ? Hook::exec('displayPaymentTop') : '')
        ));
        $this->context->smarty->assign($this->_getPaymentMethods());

        if ((bool)Configuration::get('PS_ADVANCED_PAYMENT_API')) {
            $this->context->smarty->assign(array(
                'HOOK_ADVANCED_PAYMENT' => Hook::exec('advancedPaymentOptions', array(), null, true),
                'link_conditions' => $this->link_conditions
            ));
        }
    }

    protected function _getPaymentMethods()
    {
        if (!$this->isLogged) {
            $this->warnings[] = Tools::displayError('Please sign in to see payment methods.');

            return;
        }

        if ($this->context->cart->OrderExists()) {
            $this->warnings[] = Tools::displayError('Error: This order has already been validated.');

            return;
        }

        if (!$this->context->cart->id_customer || !Customer::customerIdExistsStatic($this->context->cart->id_customer) || Customer::isBanned($this->context->cart->id_customer)) {
            $this->warnings[] = Tools::displayError('Error: No customer.');

            return;
        }

        $address_delivery = new Address($this->context->cart->id_address_delivery);
        $address_invoice = ($this->context->cart->id_address_delivery == $this->context->cart->id_address_invoice ? $address_delivery : new Address($this->context->cart->id_address_invoice));

        if (!$this->context->cart->id_address_delivery || !$this->context->cart->id_address_invoice || !Validate::isLoadedObject($address_delivery) || !Validate::isLoadedObject($address_invoice) || $address_invoice->deleted || $address_delivery->deleted) {
            $this->warnings[] = Tools::displayError('Error: Please select an address.');

            return;
        }

        if (count($this->context->cart->getDeliveryOptionList()) == 0 && !$this->context->cart->isVirtualCart()) {
            if ($this->context->cart->isMultiAddressDelivery()) {
                $this->warnings[] = Tools::displayError('Error: None of your chosen carriers deliver to some of the addresses you have selected.');

                return;
            } else {
                $this->warnings[] = Tools::displayError('Error: None of your chosen carriers deliver to the address you have selected.');

                return;
            }
        }

        if (!$this->context->cart->getDeliveryOption(null, false) && !$this->context->cart->isVirtualCart()) {
            $this->warnings[] = Tools::displayError('Error: Please choose a carrier.');

            return;
        }

        if (!$this->context->cart->id_currency) {
            $this->warnings[] = Tools::displayError('Error: No currency has been selected.');

            return;
        }

        /* If some products have disappear */
        if (is_array($product = $this->context->cart->checkQuantities(true))) {
            $this->warnings[] = sprintf(
                Tools::displayError('An item (%s) in your cart is no longer available in this quantity. You cannot proceed with your order until the quantity is adjusted.'),
                $product['name']
            );

            return;
        }

        if ((int)$id_product = $this->context->cart->checkProductsAccess()) {
            $this->warnings[] = sprintf(
                Tools::displayError('An item in your cart is no longer available (%s). You cannot proceed with your order.'),
                Product::getProductName((int)$id_product)
            );

            return;
        }

        /* Check minimal amount */
        $currency = Currency::getCurrency((int)$this->context->cart->id_currency);

        $minimal_purchase = Tools::convertPrice((float)Configuration::get('PS_PURCHASE_MINIMUM'), $currency);

        if ($this->context->cart->getOrderTotal(false, Cart::ONLY_PRODUCTS) < $minimal_purchase) {
            $this->warnings[] = sprintf(
                Tools::displayError('A minimum purchase total of %1s (tax excl.) is required to validate your order, current purchase total is %2s (tax excl.).'),
                Tools::displayPrice($minimal_purchase, $currency),
                Tools::displayPrice($this->context->cart->getOrderTotal(false, Cart::ONLY_PRODUCTS), $currency)
            );

            return;
        }

        /* Bypass payment step if total is 0 */
        if ($this->context->cart->getOrderTotal() <= 0) {
            return array(
                'HOOK_PAYMENT' => '<p class="center"><button class="button btn btn-default button-medium" name="confirmOrder" id="confirmOrder" onclick="confirmFreeOrder();" type="submit"> <span>'.Tools::displayError('I confirm my order.').'</span></button></p>',
                'freeOrder' => 1
            );
        }

        $hook_payment_array = Hook::exec('displayPayment', array(), null, true);
        $hook_payment = implode('', $hook_payment_array);
        $payment_methods = PaymentModule::getInstalledPaymentModules();

        if (!class_exists('simple_html_dom')) {
            require_once _PS_MODULE_DIR_.$this->module_name.'/libraries/simple_html_dom.php';
        }

        foreach ($payment_methods as $id => &$payment_method) {
            if (!isset($hook_payment_array[$payment_method['name']]) || $hook_payment_array[$payment_method['name']] == '') {
                unset($payment_methods[$id]);
                continue;
            }

            $display_payment = $hook_payment_array[$payment_method['name']];
            $html = str_get_html(mb_convert_encoding($display_payment, 'HTML-ENTITIES', 'UTF-8'));
            $link = $html->find('.payment_module a,
                .payment_module h1,.payment_module h1,
                .payment_module h1,.payment_module h2,
                .payment_module h1,.payment_module h3,
                .payment_module h1,.payment_module h4,
                .payment_module h1,.payment_module h5,
                .payment_module h1,.payment_module h6', 0);
            $form = $html->find('form', 0);

            $payment_method['html'] = trim($display_payment);
            $payment_method['title'] = trim((($link) ? $link->innertext : ''));
            $payment_method['link'] = html_entity_decode(trim((($link) ? $link->getAttribute('href') : '')));
            $payment_method['form'] = trim((($form) ? $form->outertext : ''));

            /* set payment method content only */
            $is_url = filter_var($payment_method['link'], FILTER_VALIDATE_URL);

            if ($is_url !== false) {
                $url = $payment_method['link'];
                $parse_url = parse_url($url);
                (!isset($parse_url['query'])) ? $parse_url['query'] = '' : false;
                parse_str($parse_url['query'], $query);
                $query['content_only'] = 1;
                $parse_url['query'] = urldecode(http_build_query($query));
                $payment_method['link'] = OrderOpcController::unparse_url($parse_url);
            } else {
                $payment_method['link'] = '';
            }
            /* (end) set payment method content only */
        }

        /* set default payment method */
        if (count($payment_methods) > 0 && !isset($this->context->cookie->id_payment)) {
            $this->context->cookie->__set('id_payment', reset($payment_methods)['name']);
        }
        /* (end) set default payment method */

        if (!$hook_payment) {
            $hook_payment = '<p class="warning">'.Tools::displayError('No payment method is available for use at this time. ').'</p>';
        }

        return array(
            'HOOK_PAYMENT' => $hook_payment,
            'payments' => $payment_methods
        );
    }

    protected function _getCarrierList()
    {
        $address_delivery = new Address($this->context->cart->id_address_delivery);
        $cms = new CMS(Configuration::get('PS_CONDITIONS_CMS_ID'), $this->context->language->id);
        $link_conditions = $this->context->link->getCMSLink($cms, $cms->link_rewrite, Configuration::get('PS_SSL_ENABLED'));

        if (!strpos($link_conditions, '?')) {
            $link_conditions .= '?content_only=1';
        } else {
            $link_conditions .= '&content_only=1';
        }

        $carriers = $this->context->cart->simulateCarriersOutput();
        $delivery_option = $this->context->cart->getDeliveryOption(null, false, false);
        $wrapping_fees = $this->context->cart->getGiftWrappingPrice(false);
        $wrapping_fees_tax_inc = $this->context->cart->getGiftWrappingPrice();
        $old_message = Message::getMessageByCartId((int)$this->context->cart->id);

        $free_shipping = false;

        foreach ($this->context->cart->getCartRules() as $rule) {
            if ($rule['free_shipping'] && !$rule['carrier_restriction']) {
                $free_shipping = true;
                break;
            }
        }

        $this->context->smarty->assign('isVirtualCart', $this->context->cart->isVirtualCart());

        $vars = array(
            'advanced_payment_api' => (bool)Configuration::get('PS_ADVANCED_PAYMENT_API'),
            'free_shipping' => $free_shipping,
            'checkedTOS' => (int)$this->context->cookie->checkedTOS,
            'recyclablePackAllowed' => (int)Configuration::get('PS_RECYCLABLE_PACK'),
            'giftAllowed' => (int)Configuration::get('PS_GIFT_WRAPPING'),
            'cms_id' => (int)Configuration::get('PS_CONDITIONS_CMS_ID'),
            'conditions' => (int)Configuration::get('PS_CONDITIONS'),
            'link_conditions' => $link_conditions,
            'recyclable' => (int)$this->context->cart->recyclable,
            'gift_wrapping_price' => (float)$wrapping_fees,
            'total_wrapping_cost' => Tools::convertPrice($wrapping_fees_tax_inc, $this->context->currency),
            'total_wrapping_tax_exc_cost' => Tools::convertPrice($wrapping_fees, $this->context->currency),
            'delivery_option_list' => $this->context->cart->getDeliveryOptionList(),
            'carriers' => $carriers,
            'checked' => $this->context->cart->simulateCarrierSelectedOutput(),
            'delivery_option' => $delivery_option,
            'address_collection' => $this->context->cart->getAddressCollection(),
            'opc' => true,
            'oldMessage' => isset($old_message['message']) ? $old_message['message'] : '',
            'HOOK_BEFORECARRIER' => Hook::exec('displayBeforeCarrier', array(
                'carriers' => $carriers,
                'delivery_option_list' => $this->context->cart->getDeliveryOptionList(),
                'delivery_option' => $delivery_option
            ))
        );

        Cart::addExtraCarriers($vars);

        $this->context->smarty->assign($vars);

        if (!Address::isCountryActiveById((int)$this->context->cart->id_address_delivery) && $this->context->cart->id_address_delivery != 0) {
            $this->errors[] = Tools::displayError('This address is not in a valid area.');
        } elseif ((!Validate::isLoadedObject($address_delivery) || $address_delivery->deleted) && $this->context->cart->id_address_delivery != 0) {
            $this->errors[] = Tools::displayError('This address is invalid.');
        } else {
            $result = array(
                'HOOK_BEFORECARRIER' => Hook::exec('displayBeforeCarrier', array(
                    'carriers' => $carriers,
                    'delivery_option_list' => $this->context->cart->getDeliveryOptionList(),
                    'delivery_option' => $this->context->cart->getDeliveryOption(null, true)
                )),
                'carrier_block' => $this->context->smarty->fetch(_PS_THEME_DIR_.'order-carrier.tpl')
            );

            Cart::addExtraCarriers($result);

            return $result;
        }

        if (count($this->errors)) {
            $this->datas['carrier_block'] = $this->context->smarty->fetch(_PS_THEME_DIR_.'order-carrier.tpl');
        }
    }

    protected function _getPaymentConfirmation()
    {
        $_SERVER['REQUEST_URI'] = '/en/module/bankwire/payment';
        unset($_GET['controller']);

        Dispatcher::$instance = null;
        $dispatcher = Dispatcher::getInstance();
        $dispatcher->getController();

        $templateVars = $this->context->smarty->getTemplateVars();

        $controller_class = Tools::getValue('module').Tools::getValue('controller').'ModuleFrontController';
        include_once(_PS_MODULE_DIR_.Tools::getValue('module').'/controllers/front/'.Tools::getValue('controller').'.php');

        $controller = new $controller_class(false, false);
        $controller->display_header = false;
        $controller->display_footer = false;
        $controller->init();
        $controller->postProcess();
        $controller->initContent();

        ob_start();
        $controller->display();
        $content = ob_get_contents();
        $content = preg_replace('/<\/body><\/html>$/', '', $content);
        ob_end_clean();

        unset($_GET['module']);
        $this->context->smarty->clearAllAssign();
        $this->context->smarty->assign($templateVars);

        return $content;
    }

    protected function _processAddressFormat()
    {
        $address_delivery = new Address((int)$this->context->cart->id_address_delivery);
        $address_invoice = new Address((int)$this->context->cart->id_address_invoice);

        $inv_adr_fields = AddressFormat::getOrderedAddressFields((int)$address_delivery->id_country, false, true);
        $dlv_adr_fields = AddressFormat::getOrderedAddressFields((int)$address_invoice->id_country, false, true);
        $require_form_fields_list = AddressFormat::getFieldsRequired();

        // Add missing require fields for a new user susbscription form
        foreach ($require_form_fields_list as $field_name) {
            if (!in_array($field_name, $dlv_adr_fields)) {
                $dlv_adr_fields[] = trim($field_name);
            }
        }

        foreach ($require_form_fields_list as $field_name) {
            if (!in_array($field_name, $inv_adr_fields)) {
                $inv_adr_fields[] = trim($field_name);
            }
        }

        foreach (array('inv', 'dlv') as $adr_type) {
            foreach (${$adr_type.'_adr_fields'} as $fields_line) {
                foreach (explode(' ', $fields_line) as $field_item) {
                    ${$adr_type.'_all_fields'}[] = trim($field_item);
                }
            }

            ${$adr_type.'_adr_fields'} = array_unique(${$adr_type.'_adr_fields'});
            ${$adr_type.'_all_fields'} = array_unique(${$adr_type.'_all_fields'});

            $this->context->smarty->assign(array(
                $adr_type.'_adr_fields' => ${$adr_type.'_adr_fields'},
                $adr_type.'_all_fields' => ${$adr_type.'_all_fields'},
                'required_fields' => $require_form_fields_list
            ));
        }
    }

    protected function _getFormatedSummaryDetail()
    {
        $result = array(
            'summary' => $this->context->cart->getSummaryDetails(),
            'customizedDatas' => Product::getAllCustomizedDatas($this->context->cart->id, null, true)
        );

        foreach ($result['summary']['products'] as &$product) {
            $product['quantity_without_customization'] = $product['quantity'];
            if ($result['customizedDatas']) {
                if (isset($result['customizedDatas'][(int)$product['id_product']][(int)$product['id_product_attribute']])) {
                    foreach ($result['customizedDatas'][(int)$product['id_product']][(int)$product['id_product_attribute']] as $addresses) {
                        foreach ($addresses as $customization) {
                            $product['quantity_without_customization'] -= (int)$customization['quantity'];
                        }
                    }
                }
            }
        }

        if ($result['customizedDatas']) {
            Product::addCustomizationPrice($result['summary']['products'], $result['customizedDatas']);
        }

        return $result;
    }

    protected function _editCustomer()
    {
        if (!($this->isLogged || $this->context->customer->is_guest)) {
            $this->errors[] = 'Please login or follow instant checkout';

            return;
        }

        $_POST['firstname'] = Tools::getValue('customer_firstname');
        $_POST['lastname'] = Tools::getValue('customer_lastname');

        /* validate and set date of birth */
        if (Tools::getValue('months') != '' && Tools::getValue('days') != '' && Tools::getValue('years') != '') {
            $this->context->customer->birthday = (int)Tools::getValue('years').'-'.(int)Tools::getValue('months').'-'.(int)Tools::getValue('days');
        } elseif (Tools::getValue('months') == '' && Tools::getValue('days') == '' && Tools::getValue('years') == '') {
            $this->context->customer->birthday = null;
        } else {
            $this->errors[] = Tools::displayError('Invalid date of birth.');
        }

        /* validate and set email */
        $email = trim(Tools::getValue('email'));
        $prev_id_default_group = 0;
        $origin_newsletter = (bool)$this->context->customer->newsletter;

        if (!Validate::isEmail($email)) {
            $this->errors[] = Tools::displayError('This email address is not valid');
        } elseif ($this->context->customer->email != $email && Customer::customerExists($email, true)) {
            $this->errors[] = Tools::displayError('An account using this email address has already been registered.');
        } else {
            $prev_id_default_group = $this->context->customer->id_default_group;
            $this->errors = array_merge($this->errors, $this->context->customer->validateController());
        }

        if (!count($this->errors)) {
            $this->context->customer->id_default_group = (int)$prev_id_default_group;
            $this->context->customer->firstname = Tools::ucwords($this->context->customer->firstname);

            if (Configuration::get('PS_B2B_ENABLE')) {
                $this->context->customer->website = Tools::getValue('website');
                $this->context->customer->company = Tools::getValue('company');
            }

            if (!Tools::getIsset('newsletter')) {
                $this->context->customer->newsletter = 0;
            } elseif (!$origin_newsletter && Tools::getIsset('newsletter')) {
                if ($module_newsletter = Module::getInstanceByName('blocknewsletter')) {
                    if ($module_newsletter->active) {
                        $module_newsletter->confirmSubscription($this->context->customer->email);
                    }
                }
            }

            if (!Tools::getIsset('optin')) {
                $this->context->customer->optin = 0;
            }

            if (Tools::getValue('passwd')) {
                $this->context->cookie->passwd = $this->context->customer->passwd;
            }

            $this->context->customer->is_guest = Tools::isSubmit('is_new_customer') ?
                !Tools::getValue('is_new_customer', 1) : 0;

            if ($this->context->customer->update()) {
                $this->context->cookie->customer_lastname = $this->context->customer->lastname;
                $this->context->cookie->customer_firstname = $this->context->customer->firstname;

                $this->datas['id_customer'] = (int)$this->context->customer->id;
                $this->datas['id_address_delivery'] = (int)$this->context->cart->id_address_delivery;
                $this->datas['id_address_invoice'] = (int)$this->context->cart->id_address_invoice;
                $this->datas['token'] = Tools::getToken(false);
            } else {
                $this->errors[] = Tools::displayError('The information cannot be updated.');
            }
        }
    }

    public static function unparse_url($parsed_url)
    {
        $scheme = isset($parsed_url['scheme']) ? $parsed_url['scheme'].'://' : '';
        $host = isset($parsed_url['host']) ? $parsed_url['host'] : '';
        $port = isset($parsed_url['port']) ? ':'.$parsed_url['port'] : '';
        $user = isset($parsed_url['user']) ? $parsed_url['user'] : '';
        $pass = isset($parsed_url['pass']) ? ':'.$parsed_url['pass'] : '';
        $pass = ($user || $pass) ? "$pass@" : '';
        $path = isset($parsed_url['path']) ? $parsed_url['path'] : '';
        $query = isset($parsed_url['query']) ? '?'.$parsed_url['query'] : '';
        $fragment = isset($parsed_url['fragment']) ? '#'.$parsed_url['fragment'] : '';

        return "$scheme$user$pass$host$port$path$query$fragment";
    }
}
