/**
 * Overrides default one page checkout with Flash Checkout
 *
 * Flash Checkout by Kahanit(https://www.kahanit.com/) is licensed under a
 * Creative Creative Commons Attribution-NoDerivatives 4.0 International License.
 * Based on a work at https://www.kahanit.com/.
 * Permissions beyond the scope of this license may be available at https://www.kahanit.com/.
 * To view a copy of this license, visit http://creativecommons.org/licenses/by-nd/4.0/.
 *
 * @author    Amit Sidhpura <amit@kahanit.com>
 * @copyright 2015 Kahanit
 * @license   http://creativecommons.org/licenses/by-nd/4.0/
 * @version   1.0.1.0
 */

var countriesNeedIDNumber = [];
var countriesNeedZipCode = [];
var states = [];

(function ($) {
    var url;
    var flashcheckout;
    var flashcheckoutSummary;
    var flashcheckoutAccount;
    var flashcheckoutDelivery;
    var flashcheckoutPayment;
    var ajaxRequestStack = 0;

    $.fn.formFieldValues = function (data) {
        var els = this.find(':input').get();

        if (arguments.length === 0) {
            // return all data
            data = {};

            $.each(els, function () {
                if (this.name && !$(this).is(':disabled') && ($(this).is(':checked')
                    || /select|textarea/i.test(this.nodeName)
                    || /text|hidden|password|email|tel/i.test(this.type))) {
                    if (data[this.name] == undefined) {
                        data[this.name] = '';
                    }

                    data[this.name] = $(this).val();
                }
            });

            return data;
        } else {
            $.each(els, function () {
                if (this.name && data[this.name]) {
                    var names = data[this.name];
                    var $this = $(this);
                    if (Object.prototype.toString.call(names) !== '[object Array]') {
                        names = [names]; //backwards compat to old version of this code
                    }
                    if (this.type == 'checkbox' || this.type == 'radio') {
                        var val = $this.val();
                        var found = false;
                        for (var i = 0; i < names.length; i++) {
                            if (names[i] == val) {
                                found = true;
                                break;
                            }
                        }
                        $this.attr("checked", found);
                    } else {
                        $this.val(names[0]);
                    }
                }
            });
            return this;
        }
    };

    $.fn.flashCheckout = function () {
        flashcheckout = $('#flashcheckout');

        attachEvents();
    };

    var attachEvents = function () {
        flashcheckoutSummary = $('#flashcheckout-summary');
        flashcheckoutAccount = $('#flashcheckout-account');
        flashcheckoutDelivery = $('#flashcheckout-delivery');
        flashcheckoutPayment = $('#flashcheckout-payment');

        /*
         Cart block events
         */

        flashcheckoutSummary.find('.cart_quantity_input').typeWatch({
            highlight: true,
            wait: 1000,
            captureLength: 0,
            callback: onUpdateQuantityChange
        });
        flashcheckoutSummary.find('.cart_quantity_up').click(onPlusQuantityClick);
        flashcheckoutSummary.find('.cart_quantity_down').click(onMinusQuantityClick);
        flashcheckoutSummary.find('.cart_quantity_delete').click(onRemoveQuantityClick);
        flashcheckoutSummary.find('#voucher').submit(onAddCouponSubmit);
        flashcheckoutSummary.find('.price_discount_delete').click(onRemoveCouponClick);
        flashcheckoutSummary.find('.voucher_name').click(function () {
            flashcheckoutSummary.find('#discount_name').val($(this).data('code'));
        });

        /*
         Account block events
         */

        flashcheckoutAccount.find('#login_form').submit(onLoginSubmit);
        flashcheckoutAccount.find('#new_account_form').submit(onRegisterSubmit);
        flashcheckoutAccount.find('#invoice_address').click(bindCheckbox);
        flashcheckoutAccount.find('#login_modal').on('show.bs.modal', function (e) {
            var data_for = $(e.relatedTarget).data('for');

            switch (data_for) {
                case 'login':
                    flashcheckoutAccount.find('#login_form').show();
                    flashcheckoutAccount.find('#new_account_form').hide();

                    break;
                case 'register':
                case 'editregister':
                    flashcheckoutAccount.find('#is_new_customer').val('1');
                    flashcheckoutAccount.find('.password.is_customer_param').show();

                    flashcheckoutAccount.find('#login_form').hide();
                    flashcheckoutAccount.find('#new_account_form').show();

                    break;
                case 'instant':
                case 'editinstant':
                    flashcheckoutAccount.find('#is_new_customer').val('0');
                    flashcheckoutAccount.find('.password.is_customer_param').hide();

                    flashcheckoutAccount.find('#login_form').hide();
                    flashcheckoutAccount.find('#new_account_form').show();

                    break;
            }
        });
        flashcheckoutAccount.find('#login_modal').on('shown.bs.modal', function () {
            $(window).trigger('resize');
        });
        flashcheckoutAccount.find('.dropdown a').click(onSelectAddressClick);
        flashcheckoutAccount.find('.useanotheraddress-forinvoice').click(onUseAnotherAddressChkBxClick);

        /*
         Delivery methods block events
         */

        if ($.fn.tooltip) flashcheckoutDelivery.find('[data-toggle="tooltip"]').tooltip();
        flashcheckoutDelivery.find('.list-group-item').click(onDeliveryMethodClick);
        flashcheckoutDelivery.find('#message').change(onMessageChange);
        flashcheckoutDelivery.find('#recyclable').click(onRecyclableClick);
        flashcheckoutDelivery.find('#gift').click(onGiftClick);
        flashcheckoutDelivery.find('#gift_message').change(onGiftMessageChange);

        /*
         Payment methods block events
         */

        flashcheckoutPayment.find('.list-group-item').click(onPaymentMethodClick);
        flashcheckoutPayment.find('#confirmorder').click(onConfirmOrderClick);
        flashcheckoutPayment.find('#confirm_modal iframe').load(function () {
            $(this).contents().find('head').append('<base target="_parent">');
            $(this).height($(this).contents().find('html').height());
        });
        flashcheckoutPayment.find('#confirm_modal').on('hide.bs.modal', onConfirmModalHide);
        flashcheckoutPayment.find('#condition').click(function (e) {
            e.preventDefault();
            flashcheckoutPayment.find('#condition_modal').modal('show');
        });
        flashcheckoutPayment.find('#condition_modal iframe').load(function () {
            $(this).contents().find('head').append('<base target="_parent">');
            $(this).height($(this).contents().find('html').height());
        });
        flashcheckoutPayment.find('#condition_modal').on('shown.bs.modal', function () {
            flashcheckoutPayment.find('#condition_modal iframe').attr('src', linkConditions);
        });

        /* states management */
        setCountries();
        bindStateInputAndUpdate();
        bindZipcode();
        bindCheckbox();
        bindUniform();

        /* refresh cart widget if exists */
        if (typeof ajaxCart !== 'undefined' && $.isFunction(ajaxCart.refresh))
            ajaxCart.refresh();

        /* vat management */
        vat_number();
        vat_number_ajax();

        flashcheckoutAccount.on('input', '#company, #company_invoice', function () {
            vat_number();
        });
    };

    /* Cart block events */

    var onUpdateQuantityChange = function (qty) {
        var name = $(this.el).attr('name'),
            old_qty = $(this.el).closest('td').find('input[name="' + name + '_hidden"]').val(),
            diff_qty,
            method;

        qty = parseInt(qty);
        old_qty = parseInt(old_qty);

        if (isNaN(qty) || isNaN(old_qty))
            return false;

        if (qty > old_qty) {
            method = 'plusQuantity';
            diff_qty = qty - old_qty;
        }
        else if (old_qty > qty) {
            method = 'minusQuantity';
            diff_qty = old_qty - qty;
        }
        else {
            return false;
        }

        onUpdateQuantity($(this.el).attr('name').replace('quantity_', ''), diff_qty, method);
    };

    var onPlusQuantityClick = function (e) {
        e.preventDefault();
        onUpdateQuantity($(this).attr('id').replace('cart_quantity_up_', ''), 1, 'plusQuantity');
    };

    var onMinusQuantityClick = function (e) {
        e.preventDefault();
        onUpdateQuantity($(this).attr('id').replace('cart_quantity_down_', ''), 1, 'minusQuantity');
    };

    var onRemoveQuantityClick = function (e) {
        e.preventDefault();

        var qty = parseInt($(this).closest('tr').find('.cart_quantity_input').val());

        if (isNaN(qty))
            return false;

        onUpdateQuantity($(this).attr('id'), qty, 'minusQuantity');
    };

    var onUpdateQuantity = function (id, qty, method) {
        var id_product,
            id_product_attribute = 0,
            id_address_delivery = 0,
            ids,
            id_customization = 0,
            data;

        ids = id.split('_');
        id_product = parseInt(ids[0]);

        if (typeof(ids[1]) !== 'undefined')
            id_product_attribute = parseInt(ids[1]);

        if (typeof(ids[2]) !== 'undefined' && ids[2] !== 'nocustom')
            id_customization = parseInt(ids[2]);

        if (typeof(ids[3]) !== 'undefined')
            id_address_delivery = parseInt(ids[3]);

        data = {
            'method': method,
            'id_product': id_product,
            'ipa': id_product_attribute,
            'id_address_delivery': id_address_delivery,
            'qty': qty
        };

        if (id_customization !== 0)
            data['id_customization'] = id_customization;

        ajaxRequest(data);
    };

    var onAddCouponSubmit = function () {
        var code = $(this).closest('form').find('#discount_name').val();

        if ($.trim(code) != '')
            ajaxRequest({
                'method': 'addCoupon',
                'code': code
            });

        return false;
    };

    var onRemoveCouponClick = function (e) {
        e.preventDefault();

        var delete_discount = $(this).attr('href').match(/deleteDiscount=\d+/g),
            id_cart_rule = delete_discount[0].split('=')[1];

        ajaxRequest({
            'method': 'removeCoupon',
            'id_cart_rule': id_cart_rule
        });
    };

    /* Account block events */

    var onLoginSubmit = function () {
        var form_data = $(this).formFieldValues();

        form_data['controller'] = 'authentication';
        form_data['SubmitLogin'] = true;

        ajaxRequest(form_data).success(function (response) {
            if (typeof response.hasError != 'undefined' && !response.hasError) {
                flashcheckoutAccount.find('#login_modal').modal('hide');
                ajaxRequest({});
            }
        });

        return false;
    };

    var onRegisterSubmit = function () {
        var opc_account_details = flashcheckoutAccount.find('#opc_account_details'),
            opc_delivery_address = flashcheckoutAccount.find('#opc_delivery_address'),
            opc_invoice_address = flashcheckoutAccount.find('#opc_invoice_address'),
            form_data_account = opc_account_details.formFieldValues(),
            form_data_delivery = opc_delivery_address.formFieldValues(),
            form_data_invoice = opc_invoice_address.formFieldValues(),
            bc_token;

        if (typeof form_data_account['opc_id_customer'] !== 'undefined' && form_data_account['opc_id_customer'] != 0) {
            form_data_account['method'] = '_editCustomer';
        } else {
            form_data_account['controller'] = 'authentication';
            form_data_account['submitAccount'] = true;
        }

        $.extend(form_data_account, form_data_delivery);

        ajaxRequest(form_data_account, false).success(function (response) {
            if (typeof response.hasError != 'undefined' && !response.hasError) {
                bc_token = response['token'];

                opc_account_details.find('#opc_id_customer').val(response['id_customer']);
                opc_account_details.find('#opc_id_address_delivery').val(response['id_address_delivery']);
                opc_delivery_address.find('#id_address').val(response['id_address_delivery']);
                /*opc_account_details.find('#opc_id_address_invoice').val(response['id_address_invoice']);
                 opc_invoice_address.find('#id_address_invoice').val(response['id_address_invoice']);*/

                /* validate and set delivery address */
                form_data_delivery = opc_delivery_address.formFieldValues();
                form_data_delivery['type'] = 'delivery';
                form_data_delivery['controller'] = 'address';
                form_data_delivery['token'] = bc_token;
                form_data_delivery['submitAddress'] = true;

                ajaxRequest(form_data_delivery, false).success(function (response) {
                    if (typeof response.hasError != 'undefined' && !response.hasError) {
                        opc_account_details.find('#opc_id_address_delivery').val(response['id_address_delivery']);
                        opc_delivery_address.find('#id_address').val(response['id_address_delivery']);
                        /*opc_account_details.find('#opc_id_address_invoice').val(response['id_address_invoice']);
                         opc_invoice_address.find('#id_address_invoice').val(response['id_address_invoice']);*/

                        /* validate and set invoice address */
                        if (typeof form_data_delivery['invoice_address'] !== 'undefined' && form_data_delivery['invoice_address']) {
                            form_data_invoice = flashcheckoutAccount.find('#opc_invoice_address').formFieldValues();
                            $.each(form_data_invoice, function (index, value) {
                                var index_temp = index.replace(/_invoice$/, '');

                                form_data_invoice[index_temp] = value;
                                delete form_data_invoice[index];
                            });
                            form_data_invoice['type'] = 'invoice';
                            form_data_invoice['controller'] = 'address';
                            form_data_invoice['token'] = bc_token;
                            form_data_invoice['submitAddress'] = true;

                            if (typeof form_data_account['opc_id_customer'] === 'undefined' || form_data_account['opc_id_customer'] == 0 ||
                                form_data_invoice['id_address'] == 0 || form_data_invoice['id_address'] == form_data_delivery['id_address'])
                                form_data_invoice['id_address'] = 0;
                            else {
                                delete form_data_invoice['type'];
                                form_data_invoice['id_address'] = form_data_account['opc_id_address_invoice'];
                                form_data_invoice['opc_id_address_invoice'] = form_data_account['opc_id_address_invoice'];
                            }

                            ajaxRequest(form_data_invoice, false).success(function (response) {
                                if (typeof response.hasError != 'undefined' && !response.hasError) {
                                    flashcheckoutAccount.find('#login_modal').modal('hide');
                                    ajaxRequest({
                                        'method': 'changeAddress',
                                        'id_address_delivery': response.id_address_delivery,
                                        'id_address_invoice': (form_data_invoice['id_address'] == 0) ? response.id_address_invoice : form_data_invoice['id_address']
                                    }, true, false);
                                }
                            });
                        } else {
                            flashcheckoutAccount.find('#login_modal').modal('hide');
                            ajaxRequest({});
                        }
                    }
                });
            }
        });

        return false;
    };

    /* Address block events */

    var onSelectAddressClick = function (e) {
        if (typeof e !== 'undefined')
            e.preventDefault();

        var id_address_delivery = flashcheckoutAccount.find('#deliveryaddress input[name="id_address"]').val(),
            id_address_invoice = flashcheckoutAccount.find('#invoiceaddress input[name="id_address"]').val(),
            type = $(this).attr('data-type'),
            id_address_new = $(this).attr('data-id');

        if (type == 'delivery' && id_address_new != id_address_delivery) {
            if (!flashcheckoutAccount.find('.useanotheraddress-forinvoice').is(':checked'))
                id_address_invoice = id_address_new;

            ajaxRequest({
                'method': 'changeAddress',
                'id_address_delivery': id_address_new,
                'id_address_invoice': id_address_invoice
            });
        }

        if (type == 'invoice' && id_address_new != id_address_invoice) {
            ajaxRequest({
                'method': 'changeAddress',
                'id_address_delivery': id_address_delivery,
                'id_address_invoice': id_address_new
            });
        }
    };

    var onUseAnotherAddressChkBxClick = function () {
        if (!$(this).is(':checked')) {
            flashcheckoutAccount.find('#invoiceaddress').hide();

            var id_address_delivery = flashcheckoutAccount.find('#deliveryaddress input[name="id_address"]').val(),
                id_address_invoice = flashcheckoutAccount.find('#invoiceaddress input[name="id_address"]').val();

            if (id_address_invoice != id_address_delivery)
                ajaxRequest({
                    'method': 'changeAddress',
                    'id_address_delivery': id_address_delivery,
                    'id_address_invoice': id_address_delivery
                });
        }
        else
            flashcheckoutAccount.find('#invoiceaddress').show();
    };

    /* Delivery methods block events */

    var onDeliveryMethodClick = function () {
        var data_name = $(this).attr('data-name'),
            data_value = $(this).attr('data-value'),
            checked = $(this).hasClass('selected');

        $(this).siblings().removeClass('selected');
        $(this).addClass('selected');

        if (!checked || arguments[0] === 'other') {
            var data = {'method': 'updateCarrier'};

            if (flashcheckoutDelivery.find('#recyclable').length > 0)
                data['recyclable'] = Number(flashcheckoutDelivery.find('#recyclable').is(':checked'));
            else
                data['recyclable'] = 0;

            if (flashcheckoutDelivery.find('#gift').length > 0)
                data['gift'] = Number(flashcheckoutDelivery.find('#gift').is(':checked'));
            else
                data['gift'] = 0;

            if (flashcheckoutDelivery.find('#gift_message').length > 0)
                data['gift_message'] = flashcheckoutDelivery.find('#gift_message').val();
            else
                data['gift_message'] = '';

            data[data_name] = data_value;
            ajaxRequest(data);
        }
    };

    var onMessageChange = function () {
        var me = $(this);

        ajaxRequest({
            'method': 'updateMessage',
            'message': me.val()
        }, false).success(function (response) {
            if (typeof response !== 'undefined' && !response.hasError && typeof response.message !== 'undefined') {
                me.val(response.message);
            }
        });
    };

    var onRecyclableClick = function () {
        simulateDeliveryMethodClick();
    };

    var onGiftClick = function () {
        flashcheckoutDelivery.find('#gift_message_div').hide();

        if ($(this).is(':checked'))
            flashcheckoutDelivery.find('#gift_message_div').show();

        simulateDeliveryMethodClick();
    };

    var onGiftMessageChange = function () {
        simulateDeliveryMethodClick();
    };

    var simulateDeliveryMethodClick = function () {
        var selected = flashcheckoutDelivery.find('.list-group-item.selected');

        onDeliveryMethodClick.apply(selected, ['other']);
    };

    /* Payment methods block events */

    var onPaymentMethodClick = function () {
        var id_payment = $(this).attr('data-name'),
            checked = $(this).hasClass('selected');

        $(this).siblings().removeClass('selected');
        $(this).addClass('selected');

        if (!checked)
            ajaxRequest({
                'method': 'updatePayment',
                'id_payment': id_payment
            });
    };

    var onConfirmOrderClick = function () {
        ajaxRequest({
            'method': 'confirmOrder',
            'agreeterms': Number(flashcheckoutPayment.find('#agreeterms').is(':checked'))
        }, false).success(function (response) {
            if (typeof response !== 'undefined' && typeof response.hasError !== 'undefined' && !response.hasError) {
                if (typeof response.order_isfree !== 'undefined' && response.order_isfree) {
                    if (isGuest)
                        document.location.href = guestTrackingUrl + '?id_order=' + encodeURIComponent(response.order_reference) +
                        '&email=' + encodeURIComponent(response.order_email);
                    else
                        document.location.href = historyUrl;

                    return;
                }

                var selected = flashcheckoutPayment.find('.list-group-item.selected'),
                    link = flashcheckoutPayment.find('.list-group-item.selected').attr('data-link');

                if (link != '') {
                    flashcheckoutPayment.find('#confirm_modal iframe').attr('src', link).show();
                    flashcheckoutPayment.find('#confirm_modal #icontent').hide();
                    flashcheckoutPayment.find('#confirm_modal').modal('show');
                } else {
                    if (selected.find('form :input').not('*[type="hidden"]').length == 0) {
                        selected.find('form').submit();
                    } else {
                        flashcheckoutPayment.find('#confirm_modal iframe').hide();
                        flashcheckoutPayment.find('#confirm_modal #icontent').html(selected.find('.list-group-item-html')).show();
                        flashcheckoutPayment.find('#confirm_modal').modal('show');
                    }
                }
            }
        });
    };

    var onConfirmModalHide = function () {
        var selected = flashcheckoutPayment.find('.list-group-item.selected'),
            list_group_item_html = flashcheckoutPayment.find('#confirm_modal #icontent .list-group-item-html');

        if (list_group_item_html.length)
            selected.append(list_group_item_html);
    };

    /* Common events */

    var ajaxRequest = function (data, reload, notify) {
        var bc_controller = 'orderopc',
            bc_token = static_token;

        if (typeof reload === 'undefined')
            reload = true;

        if (typeof notify === 'undefined')
            notify = true;

        if (typeof data.controller !== 'undefined') {
            bc_controller = data.controller;
            delete data.controller;
        }

        if (typeof data.token !== 'undefined') {
            bc_token = data.token;
            delete data.token;
        }

        url = baseUri + '?rand=' + new Date().getTime() + '&controller=' + bc_controller + '&ajax=true&token=' + bc_token + '&allow_refresh=1';

        return $.ajax({
            url: url,
            headers: {"cache-control": "no-cache"},
            type: 'POST',
            async: true,
            cache: false,
            dataType: 'json',
            data: data,
            beforeSend: function () {
                ajaxRequestStack++;
                flashcheckout.find('#flashcheckout-loading').show();
            },
            complete: function () {
                ajaxRequestStack--;
                !ajaxRequestStack ? flashcheckout.find('#flashcheckout-loading').hide() : false;
            },
            success: function (response) {
                var types = ['hasSuccess', 'hasWarning', 'hasError'];

                if (typeof response === 'undefined')
                    response = {};

                if (notify)
                    $.each(types, function (iu, vu) {
                        if (typeof response[vu] !== 'undefined' && response[vu]) {
                            var title = vu.replace(/^has/, '') + ':',
                                text = '<ul>',
                                msg_key = ((vu == 'hasError') ? 'errors' : ((vu == 'hasWarning') ? 'warnings' : 'successes')),
                                type = msg_key.replace(/e?s$/, '');

                            $.each(response[msg_key], function (ii, vi) {
                                text += '<li>' + vi + '</li>';
                            });

                            text += '</ul>';

                            new PNotify({
                                title: title,
                                text: text,
                                type: type,
                                styling: 'bootstrap3',
                                delay: 5000
                            });
                        }
                    });

                if (bc_controller == 'orderopc' && reload) {
                    flashcheckoutSummary.replaceWith(response['summary-panel']);
                    flashcheckoutDelivery.replaceWith(response['delivery-panel']);
                    flashcheckoutPayment.replaceWith(response['payment-panel']);
                    flashcheckoutAccount.replaceWith(response['account-panel']);

                    attachEvents();
                }
            }
        });
    };

    /* states management */

    var setCountries = function () {
        if (typeof countries !== 'undefined' && countries) {
            var countriesPS = [];

            for (var i in countries) {
                if (countries.hasOwnProperty(i)) {
                    var id_country = countries[i]['id_country'];

                    if (typeof countries[i]['states'] !== 'undefined' && parseInt(countries[i]['contains_states'])) {
                        countriesPS[id_country] = [];

                        for (var j in countries[i]['states'])
                            if (countries[i]['states'].hasOwnProperty(j))
                                countriesPS[parseInt(id_country)].push({
                                    'id': parseInt(countries[i]['states'][j]['id_state']),
                                    'name': countries[i]['states'][j]['name']
                                });
                    }

                    if (typeof countries[i]['need_identification_number'] !== 'undefined' && parseInt(countries[i]['need_identification_number']) > 0)
                        countriesNeedIDNumber.push(parseInt(countries[i]['id_country']));

                    if (typeof countries[i]['need_zip_code'] !== 'undefined' && parseInt(countries[i]['need_zip_code']) > 0)
                        countriesNeedZipCode[parseInt(countries[i]['id_country'])] = countries[i]['zip_code_format'];
                }
            }
        }
        states = countriesPS;
    };

    var bindCheckbox = function () {
        if ($('#invoice_address:checked').length > 0) {
            $('#opc_invoice_address').fadeIn('slow');

            if ($('#company_invoice').val() == '')
                $('#vat_number_invoice, #vat_number_block_invoice').hide();

            bindUniform();
        }
        else
            $('#opc_invoice_address').fadeOut('slow');
    };

    var bindUniform = function () {
        if (!!$.prototype.uniform)
            $("select.form-control,input[type='radio'],input[type='checkbox']").uniform();
    };

    var bindZipcode = function () {
        $(document).on('keyup', 'input[name^=postcode]', function (e) {
            var char = String.fromCharCode(e.keyCode);

            if (/[a-zA-Z]/.test(char))
                $.trim($(this).val($(this).val().toUpperCase()));
        });
    };

    var bindStateInputAndUpdate = function () {
        var suffixes = [];

        $(document).on('change', 'select[id^=id_country]', function () {
            var suffix = $(this).attr('id').replace('id_country', '');

            suffixes.push(suffix);
            updateState(suffix);
            updateNeedIDNumber(suffix);
            updateZipCode(suffix);

            if (typeof validate_field !== 'undefined')
                validateZipCode('#postcode' + suffix);
        });
        $('select[id^=id_country]').trigger('change');
        $.each(suffixes, function (index, suffix) {
            if (typeof window['idSelectedState' + suffix] !== 'undefined')
                $('.id_state' + suffix + ' option[value=' + window['idSelectedState' + suffix] + ']').prop('selected', true);
        });
    };

    var updateState = function (suffix) {
        $('#id_state' + (typeof suffix !== 'undefined' ? suffix : '') + ' option:not(:first-child)').remove();

        if (typeof countries !== 'undefined')
            var state_list = states[parseInt($('#id_country' + (typeof suffix !== 'undefined' ? suffix : '')).val())];

        if (typeof state_list !== 'undefined') {
            $(state_list).each(function (key, item) {
                $('#id_state' + (typeof suffix !== 'undefined' ? suffix : '')).append('<option value="' + parseInt(item.id) + '">' + item.name + '</option>');
            });

            $('.id_state' + (typeof suffix !== 'undefined' ? suffix : '') + ':hidden').fadeIn('slow');
            $('#id_state' + (typeof suffix !== 'undefined' ? suffix : '')).uniform();
        }
        else
            $('.id_state' + (typeof suffix !== 'undefined' ? suffix : '')).fadeOut('fast');
    };

    var updateNeedIDNumber = function (suffix) {
        var id_country = parseInt($('#id_country' + (typeof suffix !== 'undefined' ? suffix : '')).val());

        if (in_array(id_country, countriesNeedIDNumber)) {
            $('.dni' + (typeof suffix !== 'undefined' ? suffix : '') + ':hidden').fadeIn('slow');
            $('#dni').uniform();
        }
        else
            $('.dni' + (typeof suffix !== 'undefined' ? suffix : '')).fadeOut('fast');
    };

    var updateZipCode = function (suffix) {
        var id_country = parseInt($('#id_country' + (typeof suffix !== 'undefined' ? suffix : '')).val());

        if (typeof countriesNeedZipCode[id_country] !== 'undefined') {
            $('.postcode' + (typeof suffix !== 'undefined' ? suffix : '') + ':hidden').fadeIn('slow');
            $('#postcode').uniform();
        }
        else
            $('.postcode' + (typeof suffix !== 'undefined' ? suffix : '')).fadeOut('fast');
    };

    var validateZipCode = function (that) {
        if ($(that).hasClass('is_required') || $(that).val().length) {
            var result = '';

            if ($(that).attr('data-validate') == 'isPostCode') {
                var selector = '#id_country';

                selector += $(that).attr('name').replace('postcode', '');

                var id_country = $(selector + ' option:selected').val();

                if (typeof(countriesNeedZipCode[id_country]) != 'undefined' && typeof(countries[id_country]['iso_code']) != 'undefined')
                    result = window['validate_' + $(that).attr('data-validate')]($(that).val(), countriesNeedZipCode[id_country], countries[id_country]['iso_code']);
            }
            else if ($(that).attr('data-validate'))
                result = window['validate_' + $(that).attr('data-validate')]($(that).val());

            if (result)
                $(that).parent().removeClass('form-error').addClass('form-ok');
            else
                $(that).parent().addClass('form-error').removeClass('form-ok');
        }
    };

    /* vat management */

    var vat_number = function () {
        var company = $('#company'),
            company_invoice = $('#company_invoice');

        if (company.length && (company.val() != ''))
            $('#vat_number, #vat_number_block').show();
        else
            $('#vat_number, #vat_number_block').hide();

        if (company_invoice.length && (company_invoice.val() != ''))
            $('#vat_number_invoice, #vat_number_block_invoice').show();
        else
            $('#vat_number_invoice, #vat_number_block_invoice').hide();
    };

    var vat_number_ajax = function () {
        var company = $('#company');

        flashcheckoutAccount.on('change', '#id_country', function () {
            if (company.length && !company.val())
                return;

            if (typeof vatnumber_ajax_call !== 'undefined' && vatnumber_ajax_call)
                $.ajax({
                    type: 'POST',
                    headers: {"cache-control": "no-cache"},
                    url: baseDir + 'modules/vatnumber/ajax.php?id_country=' + parseInt($(this).val()) + '&rand=' + new Date().getTime(),
                    success: function (isApplicable) {
                        if (isApplicable == "1") {
                            $('#vat_area').show();
                            $('#vat_number').show();
                        }
                        else
                            $('#vat_area').hide();
                    }
                });
        });

        flashcheckoutAccount.on('change', '#id_country_invoice', function () {
            var company_invoice = $('#company_invoice');

            if (company_invoice.length && !company_invoice.val())
                return;

            if (typeof vatnumber_ajax_call !== 'undefined' && vatnumber_ajax_call)
                $.ajax({
                    type: 'POST',
                    headers: {"cache-control": "no-cache"},
                    url: baseDir + 'modules/vatnumber/ajax.php?id_country=' + parseInt($(this).val()) + '&rand=' + new Date().getTime(),
                    success: function (isApplicable) {
                        if (isApplicable == "1") {
                            $('#vat_area_invoice').show();
                            $('#vat_number_invoice').show();
                        }
                        else
                            $('#vat_area_invoice').hide();
                    }
                });
        });
    };

})(jQuery);

$(function () {
    $(document).flashCheckout();
});